==> ruby-fox-0.99.166.1.tgz: share/examples/ruby/fox/gltest.rb <==
#!/bin/env ruby

require "fox"
require "responder"
require "opengl"

include Fox

class GLTestWindow < FXMainWindow

  # How often our timer will fire (in milliseconds)
  TIMER_INTERVAL = 100

  include Responder

  # Message identifiers
  ID_CANVAS, ID_SPIN, ID_SPINFAST, ID_STOP, ID_TIMEOUT, ID_CHORE =
    enum(FXMainWindow::ID_LAST, 6)

  # Canvas has been resized
  def onConfigure(sender, sel, event)
    if @glcanvas.makeCurrent
      GL.Viewport(0, 0, @glcanvas.getWidth, @glcanvas.getHeight)
      @glcanvas.makeNonCurrent
    end
    return 1
  end

  # Canvas needs repainting
  def onPaint(sender, sel, event)
    drawScene()
    return 1
  end

  # Rotate the boxes when a timer message is received
  def onTimeout(sender, sel, ptr)
    @angle += 2.0
    if @angle > 360.0
      @angle -= 360.0
    end
    drawScene()
    @timer = getApp().addTimeout(TIMER_INTERVAL, self, ID_TIMEOUT)
    return 1
  end

  # Rotate the boxes when a chore message is received
  def onChore(sender, sel, ptr)
    @angle += 2.0
    if @angle > 360.0
      @angle -= 360.0
    end
    drawScene()
    @chore = getApp().addChore(self, ID_CHORE)
    return 1
  end

  # Start the boxes spinning
  def onCmdSpin(sender, sel, ptr)
    @spinning = true
    @timer = getApp().addTimeout(TIMER_INTERVAL, self, ID_TIMEOUT)
    return 1
  end

  # Enable or disable the "Spin Timer" button
  def onUpdSpin(sender, sel, ptr)
    @spinning ? sender.disable : sender.enable
    return 1
  end

  # Start the spinning chore
  def onCmdSpinFast(sender, sel, ptr)
    @spinning = true
    @chore = getApp().addChore(self, ID_CHORE)
    return 1
  end

  # Enable or disable the "Spin Chore" button
  def onUpdSpinFast(sender, sel, ptr)
    @spinning ? sender.disable : sender.enable
    return 1
  end

  # If the boxes are spinning, stop them
  def onCmdStop(sender, sel, ptr)
    @spinning = false
    if @timer
      getApp().removeTimeout(@timer)
      @timer = nil
    end
    if @chore
      getApp().removeChore(@chore)
      @chore = nil
    end
    return 1
  end

  # Enable or disable the "Stop" button
  def onUpdStop(sender, sel, ptr)
    @spinning ? sender.enable : sender.disable
    return 1
  end

  # Draws a simple box using the given corners
  def drawBox(xmin, ymin, zmin, xmax, ymax, zmax)
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(0.0, 0.0, -1.0)
      GL.Vertex3f(xmin, ymin, zmin)
      GL.Vertex3f(xmin, ymax, zmin)
      GL.Vertex3f(xmax, ymin, zmin)
      GL.Vertex3f(xmax, ymax, zmin)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(1.0, 0.0, 0.0)
      GL.Vertex3f(xmax, ymin, zmin)
      GL.Vertex3f(xmax, ymax, zmin)
      GL.Vertex3f(xmax, ymin, zmax)
      GL.Vertex3f(xmax, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(0.0, 0.0, 1.0)
      GL.Vertex3f(xmax, ymin, zmax)
      GL.Vertex3f(xmax, ymax, zmax)
      GL.Vertex3f(xmin, ymin, zmax)
      GL.Vertex3f(xmin, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(-1.0, 0.0, 0.0)
      GL.Vertex3f(xmin, ymin, zmax)
      GL.Vertex3f(xmin, ymax, zmax)
      GL.Vertex3f(xmin, ymin, zmin)
      GL.Vertex3f(xmin, ymax, zmin)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(0.0, 1.0, 0.0)
      GL.Vertex3f(xmin, ymax, zmin)
      GL.Vertex3f(xmin, ymax, zmax)
      GL.Vertex3f(xmax, ymax, zmin)
      GL.Vertex3f(xmax, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal3f(0.0, -1.0, 0.0)
      GL.Vertex3f(xmax, ymin, zmax)
      GL.Vertex3f(xmax, ymin, zmin)
      GL.Vertex3f(xmin, ymin, zmax)
      GL.Vertex3f(xmin, ymin, zmin)
    GL.End()
  end

  # Draw the GL scene
  def drawScene
    lightPosition = [15.0, 10.0, 5.0, 1.0]
    lightAmbient  = [ 0.1,  0.1, 0.1, 1.0]
    lightDiffuse  = [ 0.9,  0.9, 0.9, 1.0]
    redMaterial   = [ 1.0,  0.0, 0.0, 1.0]
    blueMaterial  = [ 0.0,  0.0, 1.0, 1.0]
  
    width = @glcanvas.getWidth()
    height = @glcanvas.getHeight()
    aspect = width/height
  
    # Make context current
    @glcanvas.makeCurrent()
    
    GL.Viewport(0, 0, @glcanvas.getWidth(), @glcanvas.getHeight())
  
    GL.ClearColor(1.0, 1.0, 1.0, 1.0)
    GL.Clear(GL::COLOR_BUFFER_BIT|GL::DEPTH_BUFFER_BIT)
    GL.Enable(GL::DEPTH_TEST)
    
    GL.Disable(GL::DITHER)
  
    GL.MatrixMode(GL::PROJECTION)
    GL.LoadIdentity()
    GLU.Perspective(30.0, aspect, 1.0, 100.0)
  
    GL.MatrixMode(GL::MODELVIEW)
    GL.LoadIdentity()
    GLU.LookAt(5.0, 10.0, 15.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)
  
    GL.ShadeModel(GL::SMOOTH)
    GL.Light(GL::LIGHT0, GL::POSITION, lightPosition)
    GL.Light(GL::LIGHT0, GL::AMBIENT, lightAmbient)
    GL.Light(GL::LIGHT0, GL::DIFFUSE, lightDiffuse)
    GL.Enable(GL::LIGHT0)
    GL.Enable(GL::LIGHTING)
  
    GL.Material(GL::FRONT, GL::AMBIENT, blueMaterial)
    GL.Material(GL::FRONT, GL::DIFFUSE, blueMaterial)
  
    GL.PushMatrix()
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-1, -1, -1, 1, 1, 1)
  
    GL.Material(GL::FRONT, GL::AMBIENT, redMaterial)
    GL.Material(GL::FRONT, GL::DIFFUSE, redMaterial)
  
    GL.PushMatrix()
    GL.Translated(0.0, 1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5, -0.5, -0.5, 0.5, 0.5, 0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Translated(0.0, -1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5, -0.5, -0.5, 0.5, 0.5, 0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 1.0, 0.0, 0.0)
    GL.Translated(0.0, 1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, -1.0, 0.0, 0.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 0.0, 0.0, 1.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 0.0, 0.0, -1.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PopMatrix()
  
    # Swap if it is double-buffered
    if @glvisual.isDoubleBuffer
      @glcanvas.swapBuffers
    end
    
    # Make context non-current
    @glcanvas.makeNonCurrent
  end

  def initialize(app)
    # Invoke the base class initializer
    super(app, "OpenGL Test Application", nil, nil, DECOR_ALL, 0, 0, 800, 600)

    # Set up the message map entries
    FXMAPFUNC(SEL_PAINT,     ID_CANVAS,     "onPaint")
    FXMAPFUNC(SEL_CONFIGURE, ID_CANVAS,     "onConfigure")

    FXMAPFUNC(SEL_COMMAND,   ID_SPIN,       "onCmdSpin")
    FXMAPFUNC(SEL_UPDATE,    ID_SPIN,       "onUpdSpin")

    FXMAPFUNC(SEL_COMMAND,   ID_SPINFAST,   "onCmdSpinFast")
    FXMAPFUNC(SEL_UPDATE,    ID_SPINFAST,   "onUpdSpinFast")

    FXMAPFUNC(SEL_COMMAND,   ID_STOP,       "onCmdStop")
    FXMAPFUNC(SEL_UPDATE,    ID_STOP,       "onUpdStop")

    FXMAPFUNC(SEL_TIMEOUT,   ID_TIMEOUT,    "onTimeout")
    FXMAPFUNC(SEL_CHORE,     ID_CHORE,      "onChore")

    # Construct the main window elements
    frame = FXHorizontalFrame.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0)

    # Left pane to contain the glcanvas
    glcanvasFrame = FXVerticalFrame.new(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
  
    # Label above the glcanvas
    FXLabel.new(glcanvasFrame,"OpenGL Canvas Frame",nil,JUSTIFY_CENTER_X|LAYOUT_FILL_X)
  
    # Horizontal divider line
    FXHorizontalSeparator.new(glcanvasFrame,SEPARATOR_GROOVE|LAYOUT_FILL_X)
  
    # Drawing glcanvas
    glpanel = FXVerticalFrame.new(glcanvasFrame,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0, 0,0,0,0)
  
    # A visual to drag OpenGL
    @glvisual = FXGLVisual.new(getApp(), VISUAL_DOUBLEBUFFER)
  
    # Drawing glcanvas
    @glcanvas = FXGLCanvas.new(glpanel,@glvisual,self,ID_CANVAS,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT)
  
    # Right pane for the buttons
    buttonFrame = FXVerticalFrame.new(frame,LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
  
    # Label above the buttons
    FXLabel.new(buttonFrame,"Button Frame",nil,JUSTIFY_CENTER_X|LAYOUT_FILL_X)
  
    # Horizontal divider line
    FXHorizontalSeparator.new(buttonFrame, SEPARATOR_RIDGE|LAYOUT_FILL_X)
  
    # Spin according to timer
    FXButton.new(buttonFrame, "Spin &Timer\tSpin using interval timers\nNote the app blocks until the interal has elapsed...",nil,self,ID_SPIN,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)

    # Spin according to chore
    FXButton.new(buttonFrame,"Spin &Chore\tSpin as fast as possible using chores\nNote even though the app is very responsive, it never blocks;\nthere is always something to do...",nil,self,ID_SPINFAST,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Stop spinning
    FXButton.new(buttonFrame,"&Stop Spin\tStop this mad spinning, I'm getting dizzy",nil,self,ID_STOP,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Exit button
    FXButton.new(buttonFrame,"&Exit\tExit the application",nil,getApp(),FXApp::ID_QUIT,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Make a tooltip
    FXTooltip.new(getApp())
  
    # Initialize private variables
    @spinning = false
    @chore = nil
    @timer = nil
    @angle = 0.0
  end

  # Create and initialize
  def create()
    super()
    show()
  end
end

def run
  # Construct the application
  application = FXApp.new("GLTest", "FoxTest")

  # Open the display
  application.init(ARGV)

  # Construct the main window
  GLTestWindow.new(application)

  # Create the app's windows
  application.create()

  # Run the application
  return application.run()
end

run

==> ruby-fox-0.99.174.tgz: share/examples/ruby/fox/gltest.rb <==
#!/usr/bin/env ruby

require "fox"
require "fox/responder"
require "opengl"

include Fox

class GLTestWindow < FXMainWindow

  # How often our timer will fire (in milliseconds)
  TIMER_INTERVAL = 100

  include Responder

  # Message identifiers
  ID_CANVAS, ID_SPIN, ID_SPINFAST, ID_STOP, ID_TIMEOUT, ID_CHORE =
    enum(FXMainWindow::ID_LAST, 6)

  # Canvas has been resized
  def onConfigure(sender, sel, event)
    if @glcanvas.makeCurrent
      GL.Viewport(0, 0, @glcanvas.getWidth, @glcanvas.getHeight)
      @glcanvas.makeNonCurrent
    end
  end

  # Canvas needs repainting
  def onPaint(sender, sel, event)
    drawScene()
  end

  # Rotate the boxes when a timer message is received
  def onTimeout(sender, sel, ptr)
    @angle += 2.0
    if @angle > 360.0
      @angle -= 360.0
    end
    drawScene()
    @timer = getApp().addTimeout(TIMER_INTERVAL, self, ID_TIMEOUT)
  end

  # Rotate the boxes when a chore message is received
  def onChore(sender, sel, ptr)
    @angle += 2.0
    if @angle > 360.0
      @angle -= 360.0
    end
    drawScene()
    @chore = getApp().addChore(self, ID_CHORE)
  end

  # Start the boxes spinning
  def onCmdSpin(sender, sel, ptr)
    @spinning = true
    @timer = getApp().addTimeout(TIMER_INTERVAL, self, ID_TIMEOUT)
  end

  # Enable or disable the "Spin Timer" button
  def onUpdSpin(sender, sel, ptr)
    @spinning ? sender.disable : sender.enable
  end

  # Start the spinning chore
  def onCmdSpinFast(sender, sel, ptr)
    @spinning = true
    @chore = getApp().addChore(self, ID_CHORE)
  end

  # Enable or disable the "Spin Chore" button
  def onUpdSpinFast(sender, sel, ptr)
    @spinning ? sender.disable : sender.enable
  end

  # If the boxes are spinning, stop them
  def onCmdStop(sender, sel, ptr)
    @spinning = false
    if @timer
      getApp().removeTimeout(@timer)
      @timer = nil
    end
    if @chore
      getApp().removeChore(@chore)
      @chore = nil
    end
  end

  # Enable or disable the "Stop" button
  def onUpdStop(sender, sel, ptr)
    @spinning ? sender.enable : sender.disable
  end

  # Draws a simple box using the given corners
  def drawBox(xmin, ymin, zmin, xmax, ymax, zmax)
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(0.0, 0.0, -1.0)
      GL.Vertex(xmin, ymin, zmin)
      GL.Vertex(xmin, ymax, zmin)
      GL.Vertex(xmax, ymin, zmin)
      GL.Vertex(xmax, ymax, zmin)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(1.0, 0.0, 0.0)
      GL.Vertex(xmax, ymin, zmin)
      GL.Vertex(xmax, ymax, zmin)
      GL.Vertex(xmax, ymin, zmax)
      GL.Vertex(xmax, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(0.0, 0.0, 1.0)
      GL.Vertex(xmax, ymin, zmax)
      GL.Vertex(xmax, ymax, zmax)
      GL.Vertex(xmin, ymin, zmax)
      GL.Vertex(xmin, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(-1.0, 0.0, 0.0)
      GL.Vertex(xmin, ymin, zmax)
      GL.Vertex(xmin, ymax, zmax)
      GL.Vertex(xmin, ymin, zmin)
      GL.Vertex(xmin, ymax, zmin)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(0.0, 1.0, 0.0)
      GL.Vertex(xmin, ymax, zmin)
      GL.Vertex(xmin, ymax, zmax)
      GL.Vertex(xmax, ymax, zmin)
      GL.Vertex(xmax, ymax, zmax)
    GL.End()
  
    GL.Begin(GL::TRIANGLE_STRIP)
      GL.Normal(0.0, -1.0, 0.0)
      GL.Vertex(xmax, ymin, zmax)
      GL.Vertex(xmax, ymin, zmin)
      GL.Vertex(xmin, ymin, zmax)
      GL.Vertex(xmin, ymin, zmin)
    GL.End()
  end

  # Draw the GL scene
  def drawScene
    lightPosition = [15.0, 10.0, 5.0, 1.0]
    lightAmbient  = [ 0.1,  0.1, 0.1, 1.0]
    lightDiffuse  = [ 0.9,  0.9, 0.9, 1.0]
    redMaterial   = [ 1.0,  0.0, 0.0, 1.0]
    blueMaterial  = [ 0.0,  0.0, 1.0, 1.0]
  
    width = @glcanvas.getWidth().to_f
    height = @glcanvas.getHeight().to_f
    aspect = width/height
  
    # Make context current
    @glcanvas.makeCurrent()
    
    GL.Viewport(0, 0, @glcanvas.getWidth(), @glcanvas.getHeight())
  
    GL.ClearColor(1.0, 1.0, 1.0, 1.0)
    GL.Clear(GL::COLOR_BUFFER_BIT|GL::DEPTH_BUFFER_BIT)
    GL.Enable(GL::DEPTH_TEST)
    
    GL.Disable(GL::DITHER)
  
    GL.MatrixMode(GL::PROJECTION)
    GL.LoadIdentity()
    GLU.Perspective(30.0, aspect, 1.0, 100.0)
  
    GL.MatrixMode(GL::MODELVIEW)
    GL.LoadIdentity()
    GLU.LookAt(5.0, 10.0, 15.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0)
  
    GL.ShadeModel(GL::SMOOTH)
    GL.Light(GL::LIGHT0, GL::POSITION, lightPosition)
    GL.Light(GL::LIGHT0, GL::AMBIENT, lightAmbient)
    GL.Light(GL::LIGHT0, GL::DIFFUSE, lightDiffuse)
    GL.Enable(GL::LIGHT0)
    GL.Enable(GL::LIGHTING)
  
    GL.Material(GL::FRONT, GL::AMBIENT, blueMaterial)
    GL.Material(GL::FRONT, GL::DIFFUSE, blueMaterial)
  
    GL.PushMatrix()
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-1, -1, -1, 1, 1, 1)
  
    GL.Material(GL::FRONT, GL::AMBIENT, redMaterial)
    GL.Material(GL::FRONT, GL::DIFFUSE, redMaterial)
  
    GL.PushMatrix()
    GL.Translated(0.0, 1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5, -0.5, -0.5, 0.5, 0.5, 0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Translated(0.0, -1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5, -0.5, -0.5, 0.5, 0.5, 0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 1.0, 0.0, 0.0)
    GL.Translated(0.0, 1.75, 0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, -1.0, 0.0, 0.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 0.0, 0.0, 1.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PushMatrix()
    GL.Rotated(90.0, 0.0, 0.0, -1.0)
    GL.Translated(0.0,1.75,0.0)
    GL.Rotated(@angle, 0.0, 1.0, 0.0)
    drawBox(-0.5,-0.5,-0.5,0.5,0.5,0.5)
    GL.PopMatrix()
  
    GL.PopMatrix()
  
    # Swap if it is double-buffered
    if @glvisual.isDoubleBuffer
      @glcanvas.swapBuffers
    end
    
    # Make context non-current
    @glcanvas.makeNonCurrent
  end

  def initialize(app)
    # Invoke the base class initializer
    super(app, "OpenGL Test Application", nil, nil, DECOR_ALL, 0, 0, 800, 600)

    # Set up the message map entries
    FXMAPFUNC(SEL_PAINT,     ID_CANVAS,     "onPaint")
    FXMAPFUNC(SEL_CONFIGURE, ID_CANVAS,     "onConfigure")

    FXMAPFUNC(SEL_COMMAND,   ID_SPIN,       "onCmdSpin")
    FXMAPFUNC(SEL_UPDATE,    ID_SPIN,       "onUpdSpin")

    FXMAPFUNC(SEL_COMMAND,   ID_SPINFAST,   "onCmdSpinFast")
    FXMAPFUNC(SEL_UPDATE,    ID_SPINFAST,   "onUpdSpinFast")

    FXMAPFUNC(SEL_COMMAND,   ID_STOP,       "onCmdStop")
    FXMAPFUNC(SEL_UPDATE,    ID_STOP,       "onUpdStop")

    FXMAPFUNC(SEL_TIMEOUT,   ID_TIMEOUT,    "onTimeout")
    FXMAPFUNC(SEL_CHORE,     ID_CHORE,      "onChore")

    # Construct the main window elements
    frame = FXHorizontalFrame.new(self, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0, 0,0,0,0)

    # Left pane to contain the glcanvas
    glcanvasFrame = FXVerticalFrame.new(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
  
    # Label above the glcanvas
    FXLabel.new(glcanvasFrame,"OpenGL Canvas Frame",nil,JUSTIFY_CENTER_X|LAYOUT_FILL_X)
  
    # Horizontal divider line
    FXHorizontalSeparator.new(glcanvasFrame,SEPARATOR_GROOVE|LAYOUT_FILL_X)
  
    # Drawing glcanvas
    glpanel = FXVerticalFrame.new(glcanvasFrame,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0, 0,0,0,0)
  
    # A visual to drag OpenGL
    @glvisual = FXGLVisual.new(getApp(), VISUAL_DOUBLEBUFFER)
  
    # Drawing glcanvas
    @glcanvas = FXGLCanvas.new(glpanel,@glvisual,self,ID_CANVAS,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT)
  
    # Right pane for the buttons
    buttonFrame = FXVerticalFrame.new(frame,LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,10,10)
  
    # Label above the buttons
    FXLabel.new(buttonFrame,"Button Frame",nil,JUSTIFY_CENTER_X|LAYOUT_FILL_X)
  
    # Horizontal divider line
    FXHorizontalSeparator.new(buttonFrame, SEPARATOR_RIDGE|LAYOUT_FILL_X)
  
    # Spin according to timer
    FXButton.new(buttonFrame, "Spin &Timer\tSpin using interval timers\nNote the app blocks until the interal has elapsed...",nil,self,ID_SPIN,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)

    # Spin according to chore
    FXButton.new(buttonFrame,"Spin &Chore\tSpin as fast as possible using chores\nNote even though the app is very responsive, it never blocks;\nthere is always something to do...",nil,self,ID_SPINFAST,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Stop spinning
    FXButton.new(buttonFrame,"&Stop Spin\tStop this mad spinning, I'm getting dizzy",nil,self,ID_STOP,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Exit button
    FXButton.new(buttonFrame,"&Exit\tExit the application",nil,getApp(),FXApp::ID_QUIT,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,10,10,5,5)
  
    # Make a tooltip
    FXTooltip.new(getApp())
  
    # Initialize private variables
    @spinning = false
    @chore = nil
    @timer = nil
    @angle = 0.0
  end

  # Create and initialize
  def create()
    super()
    show()
  end
end

def run
  # Construct the application
  application = FXApp.new("GLTest", "FoxTest")

  # Open the display
  application.init(ARGV)

  # Construct the main window
  GLTestWindow.new(application)

  # Create the app's windows
  application.create()

  # Run the application
  return application.run()
end

run
